import poser

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

	def __init__(self, api):

		self.api = api							# A reference to an EZAPI object	
		self.parameters = None 					# The method to interrogate the value of UI controls (this will be set by EZSkin)

		self.has_controls = True				# True if you supply custom controls
		self.unique_name = 'AP_BBHAIR'			

		# This dictionary is used to associate an EZSkin shader with a method in your classs
		# In this case, we only have one method so all are set to invoke self.MakePBR
		# The second element of the tuple determines whether or not it should be processed,
		# As selected by the user on the main EZSkin UI
		self.shaders = {
			"bb_hair" : (self.MakeBBHair, OTHERTYPE),
			"ignore" : (None, SKINTYPE),
			}

	def _PrepareRoot(self, mat, p):
		# Used to prepare the material room for a new Superfly shader
		# In this case it will return a PoserSurface root node
		# User preferences will be taken into account (use existing root or make a new one)
		# All existing roots are managed for you

		# Get original CyclesSurface root node
		n000 = mat.roots['superfly']
		if n000:
			if n000.Type() != poser.kNodeTypeCodeCYCLESSURFACE:
				n000 = None

		# Make new root
		if p('make_root', 'settings') or n000 is None:
			n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodeCYCLESSURFACE)

		return n000

	def BuilderName(self):
		# REQUIRED: returns the name of the 'shader engine' to be displayed to the user
		return 'BB Hair Shader by AP'

	def BuildShader(self, material, data, mat):

		# Parse the data
		shader = data[0]
		scattergroup = int(data[1])

		# Don't waste time if we don't have to do anything
		if shader == 'ignore': return 'ignored'

		# Stop if no PoserSurface is found
		if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

		# Configure imagemap nodes
		p = self.parameters
		if p('gc'): mat.GC_AllMaps()
		if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

		# ... and build the shader
		if shader in self.Shaders():

			# Get the method which will build the shader
			method = self.shaders[shader][METHOD]

			# Check to see if we need to use a built-in shader
			if method is None and self.api.Build() >= 68:
				builder = self.api.GetBuilder(self.Renderer())
				if builder:
					return builder.BuildShader(material, data, mat)
				else:
					return 'failed'

			else:
				# Build the shader
				if method is not None:
					method(mat, p)
					mat.RemoveOrphans()
					mat.UpdatePreview()

				return 'processed'

	def Controls(self):
		# If has_controls = True then this method must be present and return a tuple of tuples
		# (name, label, kind, minmax, default, increment=None, tooltip='', category=None)
		# See controls.txt for examples

		# Define the controls
		# (self, name, label, kind, minmax, default, increment=None, tooltip='', category=None)
		data = (
			('PMC Color Tint', 'Colour Tint', EZCOLOURP, (None), (0.250, 0.177, 0.106), None, '', 'Colours'),
			('PMC Specular Color 1', 'Specular Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, '', 'Colours'),
			('PM Pigment Density', 'Pigment Density', EZFLOAT, (-1000.0, 1000.0), 1.0, 0.01, '', 'Main'),
			('PM Proc Color Bias', 'Proc Colour Bias', EZFLOAT, (-1000.0, 1000.0), 0.5, 0.01, '', 'Main'),
			('PM Proc Color Variation', 'Proc Colour Variation', EZFLOAT, (-1000.0, 1000.0), 0.4, 0.01, '', 'Main'),
			('PM Pattern Angle', 'Horizontal Maps?', EZBOOL, (None), False, None, '', 'Main'),
			('PM Diffuse Value', 'Diffuse Value', EZFLOAT, (-1000.0, 1000.0), 0.25, 0.01, '', 'Main'),
			('PM Specular Value 1', 'Specular Value 1', EZFLOAT, (-1000.0, 1000.0), 0.15, 0.01, '', 'Main'),
			('PM Specular Value 2', 'Specular Value 2', EZFLOAT, (-1000.0, 1000.0), 0.5, 0.01, '', 'Main'),
			('PM Specular Roughness 1', 'Specular Roughness 1', EZFLOAT, (-1000.0, 1000.0), 0.25, 0.01, '', 'Main'),
			('PM Specular Roughness 2', 'Specular Roughness 2', EZFLOAT, (-1000.0, 1000.0), 0.7, 0.01, '', 'Main'),
			('PM Specular Normal 2', 'Specular Normal 2', EZFLOAT, (-1000.0, 1000.0), 0.45, 0.01, '', 'Main'),
			('PM Anisotropy', 'Anisotropy', EZFLOAT, (-1000.0, 1000.0), 0.7, 0.01, '', 'Main'),
			('PM Use Color Map', 'Use Colour Map?', EZBOOL, (None), False, None, '', 'Main'),
			('PM Color Map Saturation', 'Colour Map Saturation', EZFLOAT, (-1000.0, 1000.0), 1.0, 0.01, '', 'Main'),
			('PM Color Map Value', 'Colour Map Value', EZFLOAT, (-1000.0, 1000.0), 1.0, 0.01, '', 'Main'),
			)

		return data

	def Nix(self, mat, p):
		pass

	def Shaders(self):
		# Used internally by EZSkin
		return list(self.shaders.keys())

	def ShaderType(self, shader):
		# Used internally by EZSkin
		return self.shaders.get(shader, (None, 0))[SHADERTYPE]

	# Our single shader building method
	def SpecialFX(self, mat, p):

		# Prepare the root node
		n000 = self._PrepareRoot(mat, p)

		# For the template, we don't actually make a shader, we just print out all the ecustom control values
		ctls = self.Controls()
		for ctl in ctls:
			name = ctl[0]
			print('%s = %s' % (name, p(name)))

	def Renderer(self):
		# Return the render engine this plugin is designed to work with
		# Must be 'Firefly' or 'Superfly'
		return 'Superfly'

#The main method for creating our the material
	def MakeBBHair(self, mat, p):
		#Configure PoserSurface
		mat.ClearPoserSurface()
		mat.SetPSValue('Diffuse_Color', (1.000, 1.000, 1.000))
		mat.SetPSValue('Diffuse_Value', 0.000)
		mat.SetPSValue('Highlight_Color', (1.000, 1.000, 1.000))
		mat.SetPSValue('Highlight_Value', 0.000)
		mat.SetPSValue('Roughness', 0.100)
		mat.SetPSValue('Ambient_Color', (0.000, 0.000, 0.000))
		mat.SetPSValue('Ambient_Value', 0.000)
		mat.SetPSValue('Transparency_Max', 1.000)
		mat.SetPSValue('Transparency_Min', 0.000)
		mat.SetPSValue('Transparency_Falloff', 0.000)
		mat.SetPSValue('Translucence_Color', (1.000, 1.000, 1.000))
		mat.SetPSValue('Translucence_Value', 1.000)
		mat.SetPSValue('Reflection_Color', (0.000, 0.000, 0.000))
		mat.SetPSValue('Reflection_Value', 0.000)
		mat.SetPSValue('Refraction_Color', (0.000, 0.000, 0.000))
		mat.SetPSValue('Refraction_Value', 0.000)
		mat.SetPSValue('Bump', 1.000)
		mat.SetPSValue('Displacement', 0.000)
		mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
		mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))
		mat.SetPSValue('ReflectionLiteMult', 0.000)
		mat.SetPSValue('ReflectionKdMult', 0.000)
		mat.SetPSValue('Gradient_Bump', 0.000)
		mat.SetPSValue('Gradient_Mode', 2.000)
		mat.SetPSValue('ShadowCatchOnly', 0.000)
		mat.SetPSValue('ToonID', 295.000)
		mat.SetPSValue('Normals_Forward', 0.000)
		mat.SetPSValue('Custom_output_1', (1.000, 1.000, 1.000))
		mat.SetPSValue('Custom_output_2', (1.000, 1.000, 1.000))
		mat.SetPSValue('Custom_output_3', (1.000, 1.000, 1.000))
		mat.SetPSValue('Volume', (1.000, 1.000, 1.000))
		mat.SetPSValue('Geometry_Edge_Dist', 0.003)
		mat.SetPSValue('Geometry_Edge_Color', (0.000, 0.000, 0.000))

		# Build nodes
		n000 = mat.Root()
		n000.SetLocation(590,5)
		TNode=mat.GetMap('transparency')
		if not TNode is None:
			TranImage=TNode.InputByInternalName('Image_Source').Value()
		else:
			TranImage=''
		CNode=mat.GetMap('color')
		if not CNode is None:
			ColourImage=CNode.InputByInternalName('Image_Source').Value()
		else:
			ColourImage=''
		
		n001 = mat.AddNode('image_map', name='Transparency Map', loc=(238, 593), collapsed=False, preview=False, Image_Source=TranImage, Auto_Fit=0.000, U_Scale=1.000, V_Scale=1.000, U_Offset=0.000, V_Offset=0.000, Texture_Coords=1.000, Image_Mapped=4.000, Background=(1.000, 1.000, 1.000), Global_Coordinates=0.000, Mirror_U=0.000, Mirror_V=0.000, Texture_Strength=1.000, Filtering=4.000, )
		texture = n001.InputByInternalName('Image_Source').Texture()
		texture.SetGamma(1.0)
		n002 = mat.AddNode('blender', name='Blender', loc=(920, 15), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=1.000, )
		n003 = mat.AddNode('math_functions', name='PM:Diffuse Value', loc=(8, 471), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Diffuse Value'), Value_2=0.000, )
		n004 = mat.AddNode('math_functions', name='Add_2', loc=(920, 114), collapsed=False, preview=False, Math_Argument=1.000, Value_1=0.010, Value_2=0.010, )
		n005 = mat.AddNode('diffuse', name='Diffuse', loc=(920, 213), collapsed=False, preview=False, Diffuse_Color=(1.000, 1.000, 1.000), Diffuse_Value=1.000, Normals_Forward=0.000, )
		n006 = mat.AddNode('color_math', name='Color_Add', loc=(920, 312), collapsed=False, preview=False, Math_Argument=1.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n007 = mat.AddNode('color_math', name='Proc Color', loc=(1150, 15), collapsed=False, preview=False, Math_Argument=23.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n008 = mat.AddNode('color_math', name='Color_Mul', loc=(1150, 114), collapsed=False, preview=False, Math_Argument=3.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n009 = mat.AddNode('math_functions', name='PM:Use Color Map', loc=(238, 10), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Use Color Map'), Value_2=0.000, )
		n010 = mat.AddNode('math_functions', name='Sub', loc=(1150, 213), collapsed=False, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
		n011 = mat.AddNode('ccl_AnisotropicBsdf', name='AnisotropicBsdf', loc=(1150, 312), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Normal=(1.000, 1.000, 1.000), Tangent=(0.000, 0.000, 0.000), Roughness=1.000, Anisotropy=1.000, Rotation=0.000, Distribution=3.000, )
		n012 = mat.AddNode('ccl_AnisotropicBsdf', name='AnisotropicBsdf_2', loc=(1150, 479), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Normal=(1.000, 1.000, 1.000), Tangent=(0.000, 0.000, 0.000), Roughness=1.000, Anisotropy=1.000, Rotation=0.000, Distribution=3.000, )
		n013 = mat.AddNode('color_math', name='Color_Pow', loc=(1380, 15), collapsed=False, preview=False, Math_Argument=9.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n014 = mat.AddNode('fractal_sum', name='FractalSum', loc=(1380, 114), collapsed=False, preview=False, x_Index=20.000, y_Index=20.000, z_Index=20.000, Octaves=5.000, Bottom=0.000, Bias=1.000, Gain=1.000, Offset_X=10000.000, Offset_Y=1000.000, Offset_Z=0.000, )
		n015 = mat.AddNode('hsv', name='HSV', loc=(1380, 332), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=1.000, )
		n016 = mat.AddNode('color_math', name='Color_Mul_2', loc=(1380, 448), collapsed=False, preview=False, Math_Argument=3.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n017 = mat.AddNode('ccl_NormalMap', name='NormalMap', loc=(1380, 547), collapsed=False, preview=False, Strength=1.000, Color=(1.000, 1.000, 1.000), Space=3.000, )
		n018 = mat.AddNode('math_functions', name='PM:Specular Roughness 1', loc=(8, 768), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Specular Roughness 1'), Value_2=0.000, )
		n019 = mat.AddNode('math_functions', name='PM:Anisotropy', loc=(8, 1065), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Anisotropy'), Value_2=0.000, )
		n020 = mat.AddNode('color_math', name='Color_Mul_3', loc=(1380, 646), collapsed=False, preview=False, Math_Argument=3.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n021 = mat.AddNode('ccl_NormalMap', name='NormalMap_2', loc=(1380, 745), collapsed=False, preview=False, Strength=1.000, Color=(1.000, 1.000, 1.000), Space=3.000, )
		n022 = mat.AddNode('math_functions', name='PM:Specular Roughness 2', loc=(8, 867), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Specular Roughness 2'), Value_2=0.000, )
		n023 = mat.AddNode('simple_color', name='PMC:Color Tint', loc=(8, 10), collapsed=False, preview=False, Color=p('PMC Color Tint'), )
		n024 = mat.AddNode('math_functions', name='PM:Pigment Density', loc=(8, 75), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Pigment Density'), Value_2=0.000, )
		n025 = mat.AddNode('math_functions', name='PM:Proc Color Bias', loc=(8, 174), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Proc Color Bias'), Value_2=0.000, )
		n026 = mat.AddNode('math_functions', name='PM:Proc Color Variation', loc=(8, 273), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Proc Color Variation'), Value_2=0.000, )
		n027 = mat.AddNode('blender', name='Blender_2', loc=(1610, 15), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=1.000, )
		n028 = mat.AddNode('blender', name='Blender_3', loc=(1610, 114), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=1.000, )
		n029 = mat.AddNode('color_math', name='Color_Pow_2', loc=(1610, 213), collapsed=False, preview=False, Math_Argument=9.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n030 = mat.AddNode('math_functions', name='PM:Color Map Saturation', loc=(238, 395), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Color Map Saturation'), Value_2=0.000, )
		n031 = mat.AddNode('math_functions', name='PM:Color Map Value', loc=(238, 494), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Color Map Value'), Value_2=0.000, )
		n032 = mat.AddNode('hsv', name='Specular Color 1', loc=(1610, 312), collapsed=False, preview=False, Color=p('PMC Specular Color 1'), Hue=1.000, Saturation=1.000, Value=1.000, )
		n033 = mat.AddNode('user_defined', name='UserDefined', loc=(1610, 428), collapsed=False, preview=False, Red=0.500, Green=0.500, Blue=1.000, Color_Mode=1.000, )
		n034 = mat.AddNode('hsv', name='HSV_3', loc=(1610, 544), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=1.000, )
		n035 = mat.AddNode('user_defined', name='UserDefined_2', loc=(1610, 660), collapsed=False, preview=False, Red=0.500, Green=1.000, Blue=1.000, Color_Mode=1.000, )
		n036 = mat.AddNode('s', name='u_Texture_Coordinate', loc=(1840, 15), collapsed=False, preview=False, )
		n037 = mat.AddNode('t', name='v_Texture_Coordinate', loc=(1840, 63), collapsed=False, preview=False, )
		n038 = mat.AddNode('math_functions', name='PM:Pattern Angle', loc=(8, 372), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Pattern Angle'), Value_2=0.000, )
		n039 = mat.AddNode('math_functions', name='Sub_2', loc=(1840, 111), collapsed=False, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=1.000, )
		n040 = mat.AddNode('image_map', name='Color Map', loc=(238, 109), collapsed=False, preview=False, Image_Source=ColourImage, Auto_Fit=0.000, U_Scale=1.000, V_Scale=1.000, U_Offset=0.000, V_Offset=0.000, Texture_Coords=1.000, Image_Mapped=4.000, Background=(1.000, 1.000, 1.000), Global_Coordinates=0.000, Mirror_U=0.000, Mirror_V=0.000, Texture_Strength=1.000, Filtering=4.000, )
		n041 = mat.AddNode('math_functions', name='Mul', loc=(1840, 210), collapsed=False, preview=False, Math_Argument=3.000, Value_1=0.250, Value_2=1.000, )
		n042 = mat.AddNode('color_math', name='Color_Div', loc=(1840, 309), collapsed=False, preview=False, Math_Argument=4.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n043 = mat.AddNode('math_functions', name='PM:Specular Value 2', loc=(8, 669), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Specular Value 2'), Value_2=0.000, )
		n044 = mat.AddNode('math_functions', name='PM:Specular Normal 2', loc=(8, 966), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Specular Normal 2'), Value_2=0.000, )
		n045 = mat.AddNode('math_functions', name='PM:Specular Value 1', loc=(8, 570), collapsed=False, preview=False, Math_Argument=1.000, Value_1=p('PM Specular Value 1'), Value_2=0.000, )
		n046 = mat.AddNode('color_math', name='Color_Pow_3', loc=(2070, 15), collapsed=False, preview=False, Math_Argument=9.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
		n047 = mat.AddNode('hsv', name='HSV_4', loc=(2300, 15), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=0.000, Value=1.000, )
		n048 = mat.AddNode('math_functions', name='Add_16', loc=(2300, 131), collapsed=False, preview=False, Math_Argument=1.000, Value_1=0.250, Value_2=0.000, )

		# Connect nodes
		mat.ConnectNodes(n014, n010, 'Value_1')
		mat.ConnectNodes(n023, n013, 'Value_1')
		mat.ConnectNodes(n024, n013, 'Value_2')
		mat.ConnectNodes(n044, n035, 'Green')
		mat.ConnectNodes(n038, n039, 'Value_2')
		mat.ConnectNodes(n002, n042, 'Value_1')
		mat.ConnectNodes(n046, n042, 'Value_2')
		mat.ConnectNodes(n045, n041, 'Value_1')
		mat.ConnectNodes(n014, n041, 'Value_2')
		mat.ConnectNodes(n042, n034, 'Color')
		mat.ConnectNodes(n043, n034, 'Value')
		mat.ConnectNodes(n041, n032, 'Value')
		mat.ConnectNodes(n035, n021, 'Color')
		mat.ConnectNodes(n025, n014, 'Bias')
		mat.ConnectNodes(n026, n014, 'Gain')
		mat.ConnectNodes(n027, n014, 'Offset_X')
		mat.ConnectNodes(n028, n014, 'Offset_Y')
		mat.ConnectNodes(n002, n047, 'Color')
		mat.ConnectNodes(n040, n029, 'Value_1')
		mat.ConnectNodes(n024, n029, 'Value_2')
		mat.ConnectNodes(n047, n046, 'Value_1')
		mat.ConnectNodes(n048, n046, 'Value_2')
		mat.ConnectNodes(n001, n004, 'Value_1')
		mat.ConnectNodes(n010, n004, 'Value_2')
		mat.ConnectNodes(n033, n017, 'Color')
		mat.ConnectNodes(n029, n015, 'Color')
		mat.ConnectNodes(n030, n015, 'Saturation')
		mat.ConnectNodes(n031, n015, 'Value')
		mat.ConnectNodes(n002, n005, 'Diffuse_Color')
		mat.ConnectNodes(n003, n005, 'Diffuse_Value')
		mat.ConnectNodes(n020, n012, 'Color')
		mat.ConnectNodes(n021, n012, 'Normal')
		mat.ConnectNodes(n022, n012, 'Roughness')
		mat.ConnectNodes(n019, n012, 'Anisotropy')
		mat.ConnectNodes(n013, n008, 'Value_1')
		mat.ConnectNodes(n015, n008, 'Value_2')
		mat.ConnectNodes(n016, n011, 'Color')
		mat.ConnectNodes(n017, n011, 'Normal')
		mat.ConnectNodes(n018, n011, 'Roughness')
		mat.ConnectNodes(n019, n011, 'Anisotropy')
		mat.ConnectNodes(n011, n006, 'Value_1')
		mat.ConnectNodes(n012, n006, 'Value_2')
		mat.ConnectNodes(n036, n027, 'Input_1')
		mat.ConnectNodes(n037, n027, 'Input_2')
		mat.ConnectNodes(n038, n027, 'Blending')
		mat.ConnectNodes(n037, n028, 'Input_1')
		mat.ConnectNodes(n036, n028, 'Input_2')
		mat.ConnectNodes(n039, n028, 'Blending')
		mat.ConnectNodes(n032, n016, 'Value_1')
		mat.ConnectNodes(n001, n016, 'Value_2')
		mat.ConnectNodes(n034, n020, 'Value_1')
		mat.ConnectNodes(n001, n020, 'Value_2')
		mat.ConnectNodes(n001, n000, 'Transparency_Max')
		mat.ConnectNodes(n002, n000, 'Translucence_Color')
		mat.ConnectNodes(n003, n000, 'Translucence_Value')
		mat.ConnectNodes(n004, n000, 'Bump')
		mat.ConnectNodes(n005, n000, 'AlternateDiffuse')
		mat.ConnectNodes(n006, n000, 'AlternateSpecular')
		mat.ConnectNodes(n013, n007, 'Value_1')
		mat.ConnectNodes(n014, n007, 'Value_2')
		mat.ConnectNodes(n007, n002, 'Input_1')
		mat.ConnectNodes(n008, n002, 'Input_2')
		mat.ConnectNodes(n009, n002, 'Blending')
